#ifndef _HARK__BEAMFORMING__BF_H__INCLUDED_
#define _HARK__BEAMFORMING__BF_H__INCLUDED_ 1

#include "BF_Matrix.h"

#include <vector>

namespace BF
{
    /* BF アルゴリズム種類 */
    enum BfType{
	BfUnknown = 0, BfDS, BfWDS, BfNull, BfILSE, BfLCMV, BfGJ, BfMSNR, BfGSS, BfGICA, BfGHDSS, BfML, 
    };

    /* ICA, HDSS で用いる非線形関数の種類 */
    enum NlFuncType {
	NlFuncUnknown = 0, NlFuncTanh, NlFuncCube,
    };
    
    /* UpdateMethod */
    enum UtUpdateType {
	UtID = 0, UtID_POS, UtPOS,
    };
	
    /* GHDSSで用いる幾何制約の手法 */
    enum LcConstType {
	LcDIAG = 0, LcFULL,
    };
	
    /* GHDSSで用いる分離行列の更新方法 */
    enum UsUpdateStepType {
	UsStep = 0, UsTotal,
    };
	
    class BF
    {
    public:
	static const double BF_EPS;
    public:
	BF(int numFreq, int lowerfreq, int upperfreq, int samplingRate, double soundSpeed, UtUpdateType separationMatUpdateType);
	virtual ~BF();

    public:
	virtual BfType getBfAlgorithmType() const = 0;

	virtual void updateSources(const std::vector<SourceLocation> &sourcePositions) = 0;
	virtual void executeBeamForming(const Matrix &inputSpectrum, Matrix &outputSpectrum);
	virtual void adaptSeparationMatrices() = 0;

	// CALCは廃止
	//void calculateTransformFunctions(const std::vector<MicLocation> &micPositions, const std::vector<SourceLocation> &sourcePositions, std::vector<Matrix> &transformFunctions) const;
	
	void getTransformFunctions(const std::vector<SourceLocation> &sourcePositions, std::vector<Matrix> &transformFunctions) const;
	void setTransformFunctionDatabase(const TransformFunctionDatabase &transformFunctionDatabase);
	void newTransformFunctionDatabase();
	// 分離行列データベースのset
	void setSeparationMatricesDatabase(const Database &saparationMatricesDatabase);
	void newSeparationMatricesDatabase();

	// ID検索
	int getSourceIndexUsingID(std::vector<SourceLocation> sourcePositions, SourceLocation Position);
	// TFデータベースのNeighborsを使用したPOS検索
	int getSourceIndexUsingNeighbor(std::vector<SourceLocation> sourcePositions, SourceLocation Position);
	// TFデータベースのNeighborsを使用したPOS検索
	int getSourceIndexUsingNeighbor(std::vector<std::vector<SourceLocation> > sourcePositions, SourceLocation Position, int &type);

	// 現フレームの音源位置について、直近フレームとデータベースの音源位置インデックスリスト作成
	//	sourceIndicestype: 現フレームの各音源に対するインデックスの情報元, 0:直近フレーム, 1:DB, -1:Not Found
	// 	sourceIndices: typeで指定された情報内でのインデックス
	void getSourceIndex(std::vector<SourceLocation> currentPositions, std::vector<int> &sourceIndicestype, std::vector<int> &sourceIndices);

	// 分離行列データベースの座標を更新
	//	currentPositions: 現フレームの音源位置
	//	sourceIndicestype: 現フレームの各音源に対するインデックスの情報元, 0:直近フレーム, 1:DB, -1:Not Found
	//	sourceIndices: 現フレームの各音源に対するDBの音源位置
	void updateSeparationMatricesDatabaseSource(std::vector<SourceLocation> currentPositions, std::vector<int> sourceIndicestype, std::vector<int> sourceIndices);

	// 分離行列データベースへの保存
	void updateSeparationMatricesDatabaseAll();

	// 分離行列データベースから値を取得
	//     DBIndex: 読み込むDBのインデックス
	//     currentSourceIndex: currentSeparationMatricesの対象座標インデックス
	//     currentFreqIndex: currentSeparationMatricesの対象周波数インデックス
	//     currentSeparationMatrices: 更新中の分離行列
	bool setSeparationMatricesFromDatabase(int DBIndex, int currentSourceIndex, int currentFreqIndex, Matrix &currentSeparationMatrices);

    public:
	/*== コンストラクタで初期化するのみ ==*/
	const int    numFreq;
	const int    samplingRate;
	const double soundSpeed;
	TransformFunctionDatabase  *transformFunctionDatabase;
	int numMicChannels;
	UtUpdateType separationMatUpdateType;
	int lowerfreq;
	int upperfreq;

	/*== 動的に変動 ==*/
	/* updateSources で更新 */
	std::vector<SourceLocation> sourcePositions;
	std::vector<MicLocation>    micPositions;
	std::vector<Matrix>         transformFunctions; /* size_is(numFreq), element_is(Matrix(numMic, numSrc)) */

	/* executeBeamForming で更新 */
	std::vector<Matrix>         covarianceMatrices;  /* size_is(numFreq), element_is(Matrix(numMic, numMic)) */

	/* updateSources, adaptSeparationMatrices で更新 */
	std::vector<Matrix>         separationMatrices; /* size_is(numFreq), element_is(Matrix(numSrc, numMic)) */

	/* executeBeamForming で更新 */
	Matrix                      inputSpectrum;      /* size_is(numMic, numFreq) */
	Matrix                      outputSpectrum;     /* size_is(numSrc, numFreq) */

	/*== 分離行列データベース ==*/
	Database	*separationMatricesDatabase;	// 分離行列データベース(POS保存, InitW時に使用)、POS, ID_POS時は増加する
    };
}

#endif
